-- Lua definitions:

us_0gb = {}

local claims = {}
local filename = minetest.get_worldpath().."/0gb.us"

function us_0gb.load_claims()
	local file = io.open(filename, "r")
	if file then
		for line in file:lines() do
			if line ~= "" then
				local area = line:split(" ")
				local shared = {}
				if area[3] and area[3] ~= "*" then
					for k,v in ipairs(area[3]:split(",")) do
						shared[v] = v
					end
				end
				local claimtype
				if area[4] then
					claimtype = area[4]
				else
					claimtype = "0gb_us:landclaim"
				end
				claims[area[1]] = {owner=area[2], shared=shared, claimtype=claimtype}
			end
		end
		file:close()
	end
end

function us_0gb.save_claims()
	local file = io.open(filename, "w")
	for key,value in pairs(claims) do
		local sharedata = ""
		for k,v in pairs(value.shared) do
			sharedata = sharedata..v..","
		end
		local sharestring
		if sharedata == "" then
			sharestring = "*"
		else
			sharestring = sharedata:sub(1,-2)
		end
		file:write(key.." "..value.owner.." "..sharestring.." "..value.claimtype.."\n")
	end
	file:close()
end

function us_0gb.get_chunk(pos)
	local x = math.floor(pos.x/16)
	local y = math.floor(pos.y/16)
	local z = math.floor(pos.z/16)
	return x..","..y..","..z
end

function us_0gb.get_chunk_center(pos)
	local x = math.floor(pos.x/16)*16+7.5
	local y = math.floor(pos.y/16)*16+7.5
	local z = math.floor(pos.z/16)*16+7.5
	return {x=x,y=y,z=z}
end

function us_0gb.get_owner(pos)
	local chunk = us_0gb.get_chunk(pos)
	if claims[chunk] then
		return claims[chunk].owner
	end
end

function us_0gb.can_interact(name, pos)	
	local chunk = us_0gb.get_chunk(pos)
	return claims[chunk] == nil or claims[chunk].owner == name or claims[chunk].shared[name]
end

us_0gb.default_place = minetest.item_place
us_0gb.default_dig = minetest.node_dig

-- Redefined Lua:

function minetest.node_dig(pos, node, digger)
	local player = digger:get_player_name()
	if us_0gb.can_interact(player, pos) then
		us_0gb.default_dig(pos, node, digger)
	else
		minetest.chat_send_player(player, "Area owned by "..us_0gb.get_owner(pos))
	end
end

function minetest.item_place(itemstack, placer, pointed_thing)
	if itemstack:get_definition().type == "node" then
	owner = us_0gb.get_owner(pointed_thing.above)
	player = placer:get_player_name()
		if us_0gb.can_interact(player, pointed_thing.above) then
			return us_0gb.default_place(itemstack, placer, pointed_thing)
		else
			minetest.chat_send_player(player, "Area owned by "..owner)
		end
	else
		return us_0gb.default_place(itemstack, placer, pointed_thing)
	end
end
				
us_0gb.load_claims()
-- Load now

-- In-game additions:

minetest.register_chatcommand("landowner", {
	params = "",
	description = "tells the owner of the current map chunk",
	privs = {interact=true},
	func = function(name, param)
		local player = minetest.env:get_player_by_name(name)
		local pos = player:getpos()
		local owner = us_0gb.get_owner(pos)
		if owner then
			minetest.chat_send_player(name, "This area is owned by "..owner)
		else
			minetest.chat_send_player(name, "This area is unowned.")
		end
	end,
})

minetest.register_chatcommand("unclaim", {
	params = "",
	description = "unclaims the current map chunk",
	privs = {interact=true},
	func = function(name, param)
		local player = minetest.env:get_player_by_name(name)
		local pos = player:getpos()
		local owner = us_0gb.get_owner(pos)
		local inv = player:get_inventory()
		if owner then
			if owner == name then
				chunk = us_0gb.get_chunk(pos)
				if inv:room_for_item("main", claims[chunk].claimtype) then
					player:get_inventory():add_item("main", {name=claims[chunk].claimtype})
					claims[chunk] = nil
					us_0gb.save_claims()
					minetest.chat_send_player(name, "You renounced your claim on this area.")
				else
					minetest.chat_send_player(name, "Your inventory is full.")
				end
			else
				minetest.chat_send_player(name, "This area is owned by "..owner)
			end
		else
			minetest.chat_send_player(name, "This area is unowned.")
		end
	end,
})

minetest.register_chatcommand("sharearea", {
	params = "<name>",
	description = "shares the current map chunk with <name>",
	privs = {interact=true},
	func = function(name, param)
		local player = minetest.env:get_player_by_name(name)
		local pos = player:getpos()
		local owner = us_0gb.get_owner(pos)
		if owner then
			if owner == name and name ~= param then
				if minetest.env:get_player_by_name(param) then
					claims[us_0gb.get_chunk(pos)].shared[param] = param
					us_0gb.save_claims()
					minetest.chat_send_player(name, param.." may now edit this area.")
					minetest.chat_send_player(param, name.." has just shared an area with you.")
				else
					minetest.chat_send_player(name, param.." is not a valid player.")
				end
			else
				minetest.chat_send_player(name, "This area is owned by "..owner)
			end
		else
			minetest.chat_send_player(name, "This area is unowned.")
		end
	end,
})

minetest.register_chatcommand("unsharearea", {
	params = "<name>",
	description = "unshares the current map chunk with <name>",
	privs = {interact=true},
	func = function(name, param)
		local player = minetest.env:get_player_by_name(name)
		local pos = player:getpos()
		local owner = us_0gb.get_owner(pos)
		if owner then
			if owner == name then
				if name ~= param then
					claims[us_0gb.get_chunk(pos)].shared[param] = nil
					us_0gb.save_claims()
					minetest.chat_send_player(name, param.." may no longer edit this area.")
					minetest.chat_send_player(param, name.." has just revoked your editing privileges in an area.")
				else
					minetest.chat_send_player(name, 'Use "/unclaim" to unclaim the aria.')
				end
			else
				minetest.chat_send_player(name, "This area is owned by "..owner)
			end
		else
			minetest.chat_send_player(name, "This area is unowned.")
		end
	end,
})

minetest.register_chatcommand("mayedit", {
	params = "",
	description = "lists the people who may edit the current map chunk",
	privs = {interact=true},
	func = function(name, param)
		local player = minetest.env:get_player_by_name(name)
		local pos = player:getpos()
		local mayedit = us_0gb.get_owner(pos)
		if mayedit then
			local chunk = us_0gb.get_chunk(pos)
			for user, user in pairs(claims[chunk].shared) do
				mayedit = mayedit..", "..user
			end
			minetest.chat_send_player(name, mayedit)
		else
			minetest.chat_send_player(name, "This area is unowned.")
		end
	end,
})

function us_0gb.regester_claimnode(node, image)
	local claimnode = minetest.get_current_modname()..":"..node
	minetest.register_node(claimnode, {
		description = "Land Claim",
		tiles = {image},
		groups = {oddly_breakable_by_hand=2,not_in_creative_inventory=1},
		on_place = function(itemstack, placer, pointed_thing)
			owner = us_0gb.get_owner(pointed_thing.above)
			player = placer:get_player_name()
			if owner then
				minetest.chat_send_player(player, "This area is already owned by "..owner)
			else
				minetest.env:remove_node(pointed_thing.above)
				chunk = us_0gb.get_chunk(pointed_thing.above)
				claims[chunk] = {owner=placer:get_player_name(),shared={},claimtype=claimnode}
				us_0gb.save_claims()
				minetest.chat_send_player(claims[chunk].owner, "You now own this area.")
				itemstack:take_item()
				return itemstack
			end
		end,
	})
end

us_0gb.regester_claimnode("landclaim", "0gb.us_landclaim.png")
us_0gb.regester_claimnode("landclaim_a", "0gb.us_landclaim_gem.png")

minetest.register_entity("0gb_us:showarea",{
	on_activate = function(self, staticdata, dtime_s)
		minetest.after(16,function()
			self.object:remove()
		end)
	end,
	initial_properties = {
		hp_max = 1,
		physical = true,
		weight = 0,
		collisionbox = {-8,-8,-8,8,8,8},
		visual = "mesh",
		visual_size = {x=16.1, y=16.1},
		mesh = "0gb.us_showarea.x",
		textures = {"0gb.us_showarea.png", "0gb.us_showarea.png", "0gb.us_showarea.png", "0gb.us_showarea.png", "0gb.us_showarea.png", "0gb.us_showarea.png"}, -- number of required textures depends on visual
		colors = {}, -- number of required colors depends on visual
		spritediv = {x=1, y=1},
		initial_sprite_basepos = {x=0, y=0},
		is_visible = true,
		makes_footstep_sound = false,
		automatic_rotate = false,
	}
})

minetest.register_chatcommand("showarea", {
	params = "",
	description = "highlights the boundaries of the current protected area",
	privs = {interact=true},
	func = function(name, param)
		local player = minetest.env:get_player_by_name(name)
		local pos = player:getpos()
		local owner = us_0gb.get_owner(pos)
--		if owner then
			if us_0gb.can_interact(name, pos) then
				local entpos = us_0gb.get_chunk_center(pos)
				minetest.env:add_entity(entpos, "0gb_us:showarea")
			else
				minetest.chat_send_player(name, "This area is owned by "..owner)
			end
--[[		else
			minetest.chat_send_player(name, "This area is unowned.")
		end]]
-- (Removed at Rarkenin's request)
	end,
})

minetest.after(0,function()
	local path = minetest.get_modpath("0gb_us")
	dofile(path.."/bucket.lua")
	dofile(path.."/default.lua")
	dofile(path.."/doors.lua")
	dofile(path.."/fire.lua")
end)

